function [L, U] = my_lu(A)
%MY_LU Calcolo della fattorizzazione L*U = A. 
%
% [L, U] = my_lu(A) calcola la fattorizzzione di A come prodotto di L,
% triangolare inferiore e con diagonale uguale ad 1, ed U, triangolare
% superiore. La fattorizzazione è fatta senza pivoting. 

n = size(A, 1);

if n ~= size(A, 2)
    error('MY_LU supporta solo matrici quadrate');
end

L = eye(n);
U = A;

for i = 1 : n - 1
    % Costruiamo implicitamente la matrice elementare Li tale che il 
    % prodotto Li * U(i:end, i:end) abbia la prima colonna come multiplo di
    % e1, il primo vettore della base canonica. La matrice ha la forma 
    % Li = I + v e1^T. 
    
    % Calcoliamo v solo dalla seconda componente in poi, la prima è zero.
    v = -U(i+1:end, i) / U(i,i);
    
    % Applichiamo la matrice elementare su U(i+1:end, i+1:end); la prima
    % riga e colonna possiamo evitare di calcolarle, dato che sappiamo che
    % saranno, rispettivamente, uguali a prima e zero.
    U(i+1:end, i+1:end) = U(i+1:end, i+1:end) + v * U(i, i+1:end);
    U(i+1:end,i) = 0;
    
    % Aggiorniamo L moltiplicandola per Li^(-1), che è uguale a I - ve1^T,
    % opportunamente inserita in un'identità più grande. La particolare
    % struttura della matrice permette di effettuare questo calcolo senza
    % nessuna operazione aritmetic
    L(i+1:end,i) = -v;    
end

end
